package com.topcoder.message.im.connection;

import java.util.Map;
import java.util.Set;

import com.topcoder.message.im.Connection;

/**
 * The ConnectionManager class is used to manage the instances of the Connection implementations.
 * This is the main usage of this class, it contains named Connection instances. Note that 
 * connection names are case sensitive. Methods to add, remove, clear and get connection instances are 
 * provided to fulfill this task.
 * 
 * This class has also the role of handling instantiation and configuration of the Connection 
 * implementations using a configuration file, without writing any code. 
 * The manager reads the configuration file and dynamically creates instances of the connections. 
 * 
 * This class is a singleton. There is no reason in having multiple managers since multiple 
 * connection instances can be defined.
 * 
 * Note that the user may not always want to give names. A method is provided that generates
 * names automatically.
 */
public class ConnectionManager {
	
	/**
	 * <p>The Configuration Manager namespace used by this component.</p> 
	 */
	public static final String NAMESPACE = "com.topcoder.message.im.connection.SocketConnectionManager";
	
	/**
	 * <p>The default config manager file. This file is loaded only if the config manager namespace is not created when the component is instantiated for the first time. This gives the flexibility to use other config manager file locations or formats, essential in development of applications using many different components, with different configuration files, to avoid the nightmare of having a lot of different config files, spread everywhere.</p>
	 * 
	 */
	private static final String DEFAULT_CONFIG_FILE = "com/topcoder/message/im/connection/SocketConnectionManager.properties";
	
	/**
	 * <p>The singleton instance of this class. Created in the getInstance method.</p>
	 * 
	 */
	private static ConnectionManager instance = null;
	
	/**
	 * Mapping containing the registered connection instances. It maps names (Strings) 
	 * to instances of the Connection class. A HashMap is recommended. This attribute is 
	 * instantiated in the constructor.
	 */
	private Map connections = null;
	
	/**
	 * Counter used to generate unique names.
	 */
	private int counter = 0;
	
	/**
	 * Constructor. Reads the config file and creates connections according to the
	 * properties. 
	 * 
	 * @throws ConnectionManagerException wraps any config exceptions or missing properties   
	 */
	private ConnectionManager() throws ConnectionManagerException {        
	} 
	
	/**
	 * <p>Gets the singleton instance (singleton pattern).</p>
	 * 
	 * @return the singleton instance of this class
	 * 
	 * @throws ConnectionManagerException wraps any config exceptions or missing properties
	 */
	public static ConnectionManager getInstance() throws ConnectionManagerException {        
		return null;
	} 
	//	 ---
	
	/**
	 * <p>Adds a connection instance to the manager. The connection instance is added to the map of connection instances under the given name as key. If the connection name is used (the key already exists) then no action is taken and false is returned.</p>
	 * 
	 *  
	 * 
	 * 
	 * @param name name the connection instance name
	 * @param ipServer ipServer the connection instance
	 * @return whether the connection instance was added or not
	 * @throws NullPointerException if any argument is null
	 */
	public boolean addConnection(String name, Connection ipServer) throws NullPointerException {        
		return false;
	}
	
	/**
	 * Adds a connection instance to the manager (name generated as connection_number
	 * using the counter variable). 
	 * 
	 * @param connection the connection instance
	 * 
	 * @return the unique name for the connection
	 * 
	 * @throws NullPointerException if the argument is null
	 */
	public String addConnection(Connection connection) throws NullPointerException {        
		return null;
	} 
	
	/**
	 * <p>Removes an connection instance from the manager. The connection instance is removed from the map of connection instances given its name. If the connection instance does not exist (the key does not exist) then no action is performed and false is returned.</p>
	 * 
	 * 
	 * 
	 * @throws NullPointerException 
	 * 
	 * @return whether the connection instance was removed or not
	 * @param name name the name of the connection instance to remove
	 * @throws NullPointerException if the argument is null
	 */
	public boolean removeConnection(String name) throws NullPointerException {        
		return false;
	} 
	
	/**
	 * <p>The clearConnections method clears all the connection instances from the manager. It does so by clearing the map storing them.</p>
	 * 
	 * 
	 * 
	 */
	public void clearConnections() {        
		// your code here
	} 
	
	/**
	 * <p>Returns whether an connection instance name is registered in this manager.</p>
	 * 
	 * @param name name the name of the connection instance to be tested
	 * @return whether the connection instance name is registered or not
	 * 
	 * @throws NullPointerException if the argument is null
	 */
	public boolean containsConnection(String name) throws NullPointerException {        
		return false;
	} 
	
	/**
	 * <p>Returns the list of the connection instance names contained in the manager. It actually returns a set of keys contained in the connection instance map. Note that a copy is returned, not the set obtained from Map.keySet, nor a unmodifiable set (because it is backed by the inner key set that can change).</p>
	 * 
	 * @return the list of connection instance names (list of Strings)
	 */
	public Set getConnectionNames() {        
		return null;
	} 
	
	/**
	 * <p>Gets the connection instance that is registered in the manager given its name. If the name does not exist, null is returned</p>
	 *
	 * @param name name the name of the connection instance 
	 * @return the connection instance with the given name or null if
	 *  there isn't an instance with that name
     *
	 * @throws NullPointerException if the argument is null
	 */
	public Connection getConnection(String name) throws NullPointerException {        
		return null;
	} 
		
}
